function Bessel
%Bessel equation

%add directory which contains source files (=algorithm)
addpath(genpath(pwd))
addpath(genpath('./source'))

print_messages(false); %show messages during computation

problem.pfun=@pfun;  %function handles to p,q,w functions
problem.qfun=@qfun;
problem.wfun=@wfun;
problem.a=0;         %integration interval
problem.b=1;
problem.bcs=[1 0 1 0]; %boundary conditions: bcs(1)*y(a)+bcs(2)*p(a)*y'(a)=0 and
%bcs(3)*y(b)+bcs(4)*p(b)*y'(b)=0 
problem.partition=[];  %no partition (mesh) available at this stage

tol=1e-8;  %input tolerance
exact=(((0:1001-1)+1)*pi).^2; %exact eigenvalues

%compute first 11 eigenvalues:
[E,partition]=calculate_eigenvalues(problem,0,10,tol,true);
%display results:
number_of_meshpoints=length(partition.bp.xs)
disp('  k           E_k              exact error    estimated error   relative error')
for n=1:length(E.indices)
  disp(sprintf('%3d     %18.12f       %5.1E       %5.1E         %5.1E',E.indices(n),E.eigenvalues(n),...
      abs(E.eigenvalues(n)-exact(E.indices(n)+1)),abs(E.errors(n)),...
      abs(E.eigenvalues(n)-exact(E.indices(n)+1))/exact(E.indices(n)+1)));
end

%compute E_100
problem.partition=partition;
[E,partition]=calculate_eigenvalues(problem,100,100,tol,true);
%display results:
number_of_meshpoints=length(partition.bp.xs)
disp('  k           E_k              exact error    estimated error   relative error')
for n=1:length(E.indices)
  disp(sprintf('%3d     %18.12f       %5.1E       %5.1E         %5.1E',E.indices(n),E.eigenvalues(n),...
      abs(E.eigenvalues(n)-exact(E.indices(n)+1)),abs(E.errors(n)),...
      abs(E.eigenvalues(n)-exact(E.indices(n)+1))/exact(E.indices(n)+1)));
end

%plot the eigenfunction
figure
[x,v,vp]=calculate_eigenfunction(partition,E.eigenvalues(1));
plot(x,v,'-k.')
xlabel('x')
legend('y_{100}')
axis tight

%compute E_500
problem.partition=partition;
[E,partition]=calculate_eigenvalues(problem,500,500,tol,true);
%display results:
number_of_meshpoints=length(partition.bp.xs)
disp('  k           E_k              exact error    estimated error   relative error')
for n=1:length(E.indices)
  disp(sprintf('%3d     %18.12f       %5.1E       %5.1E         %5.1E',E.indices(n),E.eigenvalues(n),...
      abs(E.eigenvalues(n)-exact(E.indices(n)+1)),abs(E.errors(n)),...
      abs(E.eigenvalues(n)-exact(E.indices(n)+1))/exact(E.indices(n)+1)));
end




%coefficient functions
function res=pfun(x)
res=x;

function res=qfun(x)
res=(1/4)./x; %vectorized!

function res=wfun(x)
res=x;


