function coulomb
%Coulomb potential, defined over an infinite integration interval
%singular in the origin.
addpath(genpath(pwd))
addpath(genpath('./source'))

print_messages(true);

problem.pfun=@pfun; %function handles to the p,q,w functions
problem.qfun=@qfun;
problem.wfun=@wfun;
problem.a=0;        %infinite integration interval
problem.b=inf;
problem.bcs=[1 0 1 0]; %boundary conditions
problem.partition=[];

tol=1e-8;
exact=-1./(4*(1:200).^2); %exact eigenvalues

%compute first eigenvalues 
[E,partition]=calculate_eigenvalues(problem,0,10,tol,true);
number_of_meshpoints=length(partition.bp.xs)
%display results
disp('first 11 eigenvalues:')
disp('  k           E_k              exact error    estimated error ')
for n=1:length(E.indices)
  disp(sprintf('%3d     %18.12f       %5.1E       %5.1E     ',E.indices(n),E.eigenvalues(n),...
      abs(E.eigenvalues(n)-exact(E.indices(n)+1)),abs(E.errors(n))));
end
% 
%compute some higher eigenvalues
[E,partition]=calculate_eigenvalues(problem,100,102,tol,true);
disp('some higher eigenvalues:')
disp('  k           E_k              exact error    estimated error')
%dsiplay results
for n=1:length(E.indices)
  disp(sprintf('%3d     %18.12f       %5.1E       %5.1E     ',E.indices(n),E.eigenvalues(n),...
      abs(E.eigenvalues(n)-exact(E.indices(n)+1)),abs(E.errors(n))));
end
%
%compute eigenvalues between -0.003 and -0.002
[E,partition]=calculate_eigenvalues(problem,-0.003,-0.002,tol,false);
disp('eigenvalues between -0.003 and -0.002')
disp('  k           E_k              exact error    estimated error')
%dsiplay results
for n=1:length(E.indices)
  disp(sprintf('%3d     %18.12f       %5.1E       %5.1E     ',E.indices(n),E.eigenvalues(n),...
      abs(E.eigenvalues(n)-exact(E.indices(n)+1)),abs(E.errors(n))));
end



%coefficient functions
function res=pfun(x)
res=1+0.*x;

function res=qfun(x)
res=-1./x;

function res=wfun(x)
res=1+0.*x;
