function harm
%harmonic oscillator

addpath(genpath(pwd))
addpath(genpath('./source'))

tol=1e-8;
exact=2*(0:1000)+1;

print_messages(true);

problem.pfun=@pfun;
problem.qfun=@qfun;
problem.wfun=@wfun;
problem.a=-inf;      %infinite integration interval
problem.b=inf;
problem.bcs=[1 0 1 0];
problem.partition=[];

%compute first eigenvalues
[E,partition]=calculate_eigenvalues(problem,0,10,tol,true);

%display results
number_of_meshpoints=length(partition.bp.xs)
disp('first 11 eigenvalues')
disp('  k           E_k              exact error    estimated error   relative error')
for n=1:length(E.indices)
  disp(sprintf('%3d     %18.12f       %5.1E       %5.1E         %5.1E',E.indices(n),E.eigenvalues(n),...
      abs(E.eigenvalues(n)-exact(E.indices(n)+1)),abs(E.errors(n)),...
      abs(E.eigenvalues(n)-exact(E.indices(n)+1))/exact(E.indices(n)+1)));
end

%show some eigenfunctions
figure
[x,v,vp]=calculate_eigenfunction(partition,E.eigenvalues(1));
plot(x,v)
legend('y_{0}')
figure
[x,v,vp]=calculate_eigenfunction(partition,E.eigenvalues(10));
plot(x,v)
legend('y_{9}')

%compute some higher eigenvalues
problem.partition=partition;
[E,partition]=calculate_eigenvalues(problem,100,110,tol,true);
%display results
number_of_meshpoints=length(partition.bp.xs)
disp('eigenvalues E_100 until E_110:')
disp('  k           E_k              exact error    estimated error   relative error')
for n=1:length(E.indices)
  disp(sprintf('%3d     %18.12f       %5.1E       %5.1E         %5.1E',E.indices(n),E.eigenvalues(n),...
      abs(E.eigenvalues(n)-exact(E.indices(n)+1)),abs(E.errors(n)),...
      abs(E.eigenvalues(n)-exact(E.indices(n)+1))/exact(E.indices(n)+1)));
end

%show some eigenfunctions
figure
[x,v,vp]=calculate_eigenfunction(partition,E.eigenvalues(1));
plot(x,v)
legend('y_{100}')

%compute eigenvalues between 250 and 300
problem.partition=partition;
[E,partition]=calculate_eigenvalues(problem,250,300,tol,false);
%display results
number_of_meshpoints=length(partition.bp.xs)
disp('eigenvalues between 250 and 300')
disp('  k           E_k              exact error    estimated error   relative error')
for n=1:length(E.indices)
  disp(sprintf('%3d     %18.12f       %5.1E       %5.1E         %5.1E',E.indices(n),E.eigenvalues(n),...
      abs(E.eigenvalues(n)-exact(E.indices(n)+1)),abs(E.errors(n)),...
      abs(E.eigenvalues(n)-exact(E.indices(n)+1))/exact(E.indices(n)+1)));
end

%coefficient functions
function res=pfun(x)
res=1;

function res=qfun(x)
res=x.^2;

function res=wfun(x)
res=1+0.*x;

