function coffey
%Coffey-Evans problem
%Reference: J.D. Pryce, IMA J. Numer. Anal. 6 (1986).
%there are close eigenvalue triplets {E2,E3,E4},
%{E6,E7,E8},... with the other eigenvalues well separated. E0 is very close to zero.

%add subdirectory to path
addpath(genpath(pwd))
addpath(genpath('./source'))

%"exact" eigenvalues 
exact=[0.0000000000000000 117.9463076620687587 231.6649292371271088 231.6649293129610125 ...
   231.6649293887949167 340.8882998096130157 445.2830895824354620 445.2831723066726394 ...
   445.2832550313310036 544.4183851493601107 637.6822498740469991 637.7043623416562739 ...
   637.7265023185114038 724.2576813500204467 800.8313121817070878 802.4787986926240517 ...
   804.2793051688604890 868.9602228707721472 909.4810465074137300 925.9773098347319556 ...
   951.8788067965913828];

tol=1e-8; %input tolerance

print_messages(true);

problem.pfun=@pfun; %function handles to p,q,w functions
problem.qfun=@qfun;
problem.wfun=@wfun;
problem.a=-pi/2;  %integration interval
problem.b=pi/2;
problem.bcs=[1 0 1 0]; %boundary conditions
problem.partition=[];

%compute first 11 eigenvalues 
[E,partition]=calculate_eigenvalues(problem,0,15,tol,true);
%display results:
number_of_meshpoints=length(partition.bp.xs)
disp('  k           E_k              exact error    estimated error   relative error')
for n=1:length(E.indices)
  disp(sprintf('%3d     %18.12f       %5.1E       %5.1E         %5.1E',E.indices(n),E.eigenvalues(n),...
      abs(E.eigenvalues(n)-exact(E.indices(1)+n)),abs(E.errors(n)),...
      abs(E.eigenvalues(n)-exact(E.indices(1)+n))/exact(E.indices(1)+n)));
end

% Compute eigenfunction y_10
[x,v,vp]=calculate_eigenfunction(partition,exact(10));
figure
plot(x,v)
hold on
plot(x,vp,'r')
hold off
legend('y_{10}','y''_{10}')
axis tight

%compute eigenvalues between 500 and 900
[E,partition]=calculate_eigenvalues(problem,500,900,tol,false);
%display results:
number_of_meshpoints=length(partition.bp.xs)
disp('  k           E_k              exact error    estimated error   relative error')
for n=1:length(E.indices)
  disp(sprintf('%3d     %18.12f       %5.1E       %5.1E         %5.1E',E.indices(n),E.eigenvalues(n),...
      abs(E.eigenvalues(n)-exact(E.indices(1)+n)),abs(E.errors(n)),...
      abs(E.eigenvalues(n)-exact(E.indices(1)+n))/exact(E.indices(1)+n)));
end



%coefficient functions
function res=pfun(x)
res=1;

function res=qfun(x)
res=-2.*30.*cos(2*x)+30^2*sin(2*x).^2;

function res=wfun(x)
res=1;


