function hydrogen
%hydrogen atom equation
%infinite integration interval, singular in the origin
addpath(genpath(pwd))
addpath(genpath('./source'))


tol=1e-6; %user input tolerance
exact=-1./(2*(0:10000)+4).^2; %exact eigenvalues

problem.pfun=@pfun;  %function handles to p,q,w functions
problem.qfun=@qfun;
problem.wfun=@wfun;
problem.a=0;        %integration interval
problem.b=inf;
problem.bcs=[1 0 1 0]; 
problem.partition=[];

print_messages(false)

%compute first eigenvalues
[E,partition]=calculate_eigenvalues(problem,0,10,tol,true);
disp('  k           E_k              exact error    estimated error')
for n=1:length(E.indices)
  disp(sprintf('%3d     %18.12f       %5.1E     %5.1E',E.indices(n),E.eigenvalues(n),...
      abs(E.eigenvalues(n)-exact(E.indices(n)+1)),abs(E.errors(n))));
end
number_of_meshpoints=length(partition.bp.xs)


%plot an eigenfunction
figure
[x,v,vp]=calculate_eigenfunction(partition,E.eigenvalues(10));
plot(x,v,'k')
xl=xlabel('x');
legend('y_{9}')
 axis tight

%compute eigenvalue E_{1000}
[E,partition]=calculate_eigenvalues(problem,1000,1000,tol,true);
disp('  k           E_k              exact error    estimated error')
for n=1:length(E.indices)
  disp(sprintf('%3d     %18.12f       %5.1E     %5.1E',E.indices(n),E.eigenvalues(n),...
      abs(E.eigenvalues(n)-exact(E.indices(n)+1)),abs(E.errors(n))));
end

number_of_meshpoints=length(partition.bp.xs)

%compute eigenvalue E_{10000}
[E,partition]=calculate_eigenvalues(problem,10000,10000,tol,true);
disp('  k           E_k              exact error    estimated error')
for n=1:length(E.indices)
  disp(sprintf('%3d     %18.12f       %5.1E     %5.1E',E.indices(n),E.eigenvalues(n),...
      abs(E.eigenvalues(n)-exact(E.indices(n)+1)),abs(E.errors(n))));
end




function res=pfun(x)
res=1+0.*x;

function res=qfun(x)
res=-1./x+2./x.^2;

function res=wfun(x)
res=1+0.*x;
