function [x,v,vp]=calculate_eigenfunction(partition,E)
%CALCULATE_EIGENFUNCTION
%input: partition: information on the mesh 
%                  = an output argument of calculate_eigenvalues 
%       E: the value of the eigenvalue 
%output: x: vector of x-values
%        v: vector of y-values in x
%        vp: vector of y'-values in x

part=partition.rp;
q0t=part.q0; p0t=part.p0; w0t=part.w0;
q1t=part.q1; p1t=part.p1; w1t=part.w1;
q2t=part.q2; p2t=part.p2; w2t=part.w2;

inda=max(part.inda,part.inf_a);
indb=max(part.indb,part.inf_b);
[m,imatch]=max((E*part.w0(inda+1:end-indb)-part.q0(inda+1:end-indb)).*part.p0(inda+1:end-indb));

%boundary condition in a:
if part.inda>0
    if E*part.w0(1)-part.q0(1)<0
        yi=[0;1];
    elseif abs(part.p0(1))> abs(E*part.w0(1)-part.q0(1)) 
        yi=[-1;0];
    else
        yi=[0;1];
    end
else
   ma=max(abs(part.bcs(1)),abs(part.bcs(2)));
   yi=[-part.bcs(2)/ma;part.bcs(1)/ma]; 
end
y=yi;
v=y(1); vp=y(2);
xs=part.xs;
%propagation from a to matching point
for i=1:imatch
  if i==part.inf_a  
     h=-1/xs(2)+1/xs(1);   
  else    
     h=xs(i+1)-xs(i);  
  end
  [T,rho(i)]=get_propagation_matrix(E,h,p0t(i),p1t(i),p2t(i),q0t(i),q1t(i),q2t(i),w0t(i),w1t(i),w2t(i));
  y=T*y;
  v=[v y(1)]; vp=[vp y(2)];  
end
% boundary condition in b:
if part.indb>0
    if E*part.w0(end)-part.q0(end)<0
        yf=[0;1];
    elseif abs(part.p0(end))> abs(E*part.w0(end)-part.q0(end))
        yf=[-1;0];
    else
        yf=[0;1];
    end
else
  ma=max(abs(part.bcs(3)),abs(part.bcs(4)));
  yf=[-part.bcs(4)/ma;part.bcs(3)/ma];
end
y=yf;
v2=y(1); vp2=y(2);
%propagation from b to matching point
for i=length(p0t):-1:imatch+1; 
   h=xs(i+1)-xs(i); 
   if i==length(p0t) && part.inf_b
       h=-1/xs(i+1)+1/xs(i);
   end
   [T,rho(i)]=get_propagation_matrix(E,h,p0t(i),p1t(i),p2t(i),q0t(i),q1t(i),q2t(i),w0t(i),w1t(i),w2t(i));
   iT(1,1)=T(2,2);  
   iT(1,2)=-T(1,2);
   iT(2,1)=-T(2,1);
   iT(2,2)=T(1,1);   
   y=iT*y; 
   v2=[y(1) v2]; vp2=[y(2) vp2];
end

%construct eigenfunction:
cp=[1 cumprod(rho(imatch:-1:1))];
v=[v./cp(end:-1:1)]; vp=[vp./cp(end:-1:1)]; 
if abs(v2(1))>eps
  mtch=v(end)/v2(1);
else
  mtch=vp(end)/vp2(1);  
end
cp=[1 cumprod(rho(imatch+1:end))];
v2=mtch*v2(1:end)./cp; vp2=mtch*vp2(1:end)./cp;
v=[v v2(2:end)];
vp=[vp vp2(2:end)];
x=part.xs;

if isinf(x(end))
    v=v(1:end-1); x=x(1:end-1); vp=vp(1:end-1);
end
end

