function [Ts,rho]=get_propagation_matrix(E,h,p0,p1,p2,q0,q1,q2,w0,w1,w2)
%GET_PROPAGATION_MATRIX
%returns transfer matrices 

Ts=zeros(2,2,length(h)); 
rho=zeros(1,length(h));
qw0=(q0-E*w0); 
qw1=(q1-E*w1); qw2=(q2-E*w2);
Z=(qw0.*p0).*h.*h;

%--------------------------------------------------
%avoid overflow for really large Z
indices1=find(Z>4e2);
%x=cosh(sqrt(Z));  et=sinh(sqrt(Z))/sqrt(Z); 
if ~isempty(indices1)
 x=ones(1,length(indices1));   
 Zt=Z(indices1); ht=h(indices1);
 et=(1-2./exp(2*sqrt(Zt)+1))./sqrt(Zt); 
 sq=sqrt(4*et.*Zt.*et)/2;
 eigv1=x+sq; eigv2=x-sq;
 rho(indices1)=max(abs(eigv1),abs(eigv2)); 
 Ts(1,1,indices1)=x./rho(indices1); Ts(1,2,indices1)=ht.*et.*p0(indices1)./rho(indices1); 
 Ts(2,1,indices1)=Zt.*et./ht./p0(indices1)./rho(indices1); Ts(2,2,indices1)=Ts(1,1,indices1);
end

%---------------------------------------------------
%normal case
indices2=find(abs(Z)>0.1 & Z<=4e2);
if ~isempty(indices2)
 x=[]; et=[];   
 Zt=Z(indices2);
 tmp=find(Zt<0);
 s=sqrt(abs(Zt(tmp)));
 x(tmp)=cos(s);     
 et(tmp)=sin(s)./s;
 tmp=find(Zt>=0);
 s=sqrt(Zt(tmp));
 x(tmp)=cosh(s);
 et(tmp)=sinh(s)./s;
 p0t=p0(indices2);
 ht=h(indices2);
 qw0t=qw0(indices2);

S1=qw0t.*p1(indices2)-p0t.*qw1(indices2); 
S2=qw0t.*p2(indices2)-p0t.*qw2(indices2);
F1=qw0t.*p1(indices2)+p0t.*qw1(indices2); 
F2=qw0t.*p2(indices2)+p0t.*qw2(indices2);
Zt2=Zt.*Zt; Zt3=Zt2.*Zt; Zt4=Zt3.*Zt;
S1t2=S1.*S1; S2t2=S2.*S2; 


%first two terms in Neumann series:
xsi=2*x.^2-1; eta=et.*x;
tmp=((-2*eta+xsi+1).*S1+(-1-6*eta+xsi).*S2)./Zt+(-3+3*xsi).*S2./Zt2; 
A11=1+tmp/4;
A22=1-tmp/4;
tmp=2*S2.*eta+2*S1.*eta+((-xsi+1).*S1+(6*eta-3-3*xsi).*S2)./Zt;
tmp=tmp./(4*ht);
A12=tmp./qw0t;
A21=-tmp./p0t;

%second integral in Neumann series:
  tmp=(((4-4*xsi+24*eta).*S1+(-4*xsi+56*eta-4).*S2).*F1+...
        ((-4-4*xsi+40*eta).*S1+(4-4*xsi+72*eta).*S2).*F2+...
        (2*xsi-8*eta+2).*S1t2+(24*eta+2-2*xsi).*S2t2)./Zt2+...
        (((12-12*xsi).*S1+(-72-72*xsi+144*eta).*S2).*F1+...
        ((-48*xsi-48+96*eta).*S1+(720*eta-156*xsi+156).*S2).*F2+...
        (2*xsi-2).*S1t2+(-30*xsi-6+72*eta).*S2t2)./Zt3+((-360*xsi+360).*S2.*F2+(18-18*xsi).*S2t2)./Zt4;  
  A11=A11+tmp/32;
  tmp=((-6./Zt2.*eta+(-18*eta+9+9*xsi)./Zt3).*S1+(-6./Zt2.*eta+(21*xsi-108*eta-21)./Zt3+...
        (54*xsi-54)./Zt4).*S2).*F1+((-6./Zt2.*eta+(15*xsi-72*eta-15)./Zt3+(-36+36*xsi)./Zt4).*S1+...
        (-6./Zt2.*eta+(27*xsi+27-234*eta)./Zt3+(-540*eta+270+270*xsi)./Zt4).*S2).*F2+((-3*eta-1)./Zt2+(3*xsi-3*eta)./Zt3).*S1t2+...
        ((3*eta-3/5)./Zt2+(-9*xsi+45*eta)./Zt3+(27*eta-27*xsi)./Zt4).*S2t2; 
  A12=A12+tmp.*ht.*p0t/24;
  tmp=((6./Zt.*eta+(-9-9*xsi+18*eta)./Zt2).*S1+(6./Zt.*eta+(108*eta+21-21*xsi)./Zt2+(-54*xsi+54)./Zt3).*S2).*F1+((6./Zt.*eta+...
      (72*eta-15*xsi+15)./Zt2+(36-36*xsi)./Zt3).*S1+...
       (6./Zt.*eta+(234*eta-27-27*xsi)./Zt2+(-270*xsi+540*eta-270)./Zt3).*S2).*F2+((-3*eta-1)./Zt+(3*xsi-3*eta)./Zt2).*S1t2+...
       ((3*eta-3/5)./Zt+(-9*xsi+45*eta)./Zt2+(27*eta-27*xsi)./Zt3).*S2t2;
  A21=A21+tmp/24./p0t./ht; 
  tmp=(((-4-24*eta+4*xsi)./Zt2+(-12+12*xsi)./Zt3).*S1+((-56*eta+4*xsi+4)./Zt2+(72-144*eta+72*xsi)./Zt3).*S2).*F1+...
        (((4-40*eta+4*xsi)./Zt2+(48*xsi+48-96*eta)./Zt3).*S1+((-72*eta-4+4*xsi)./Zt2+(156*xsi-156-720*eta)./Zt3+(-360+360*xsi)./Zt4).*S2).*F2+...
        ((2*xsi-8*eta+2)./Zt2+(-2+2*xsi)./Zt3).*S1t2+((24*eta+2-2*xsi)./Zt2+(-6-30*xsi+72*eta)./Zt3+(-18*xsi+18)./Zt4).*S2t2;
  A22=A22+tmp/32;  


sq=sqrt(4*et.*Zt.*et)/2;
eigv1=x+sq;eigv2=x-sq;

rho(indices2)=max(abs(eigv1),abs(eigv2)); %spectral radius
Ts(1,1,indices2)=(x.*A11+ht.*et.*p0t.*A21)./rho(indices2); %T=exp(h{\bar A})*(I+N1+N2)
Ts(1,2,indices2)=(x.*A12+ht.*et.*p0t.*A22)./rho(indices2);
Ts(2,1,indices2)=(Zt.*et./ht./p0t.*A11+x.*A21)./rho(indices2);
Ts(2,2,indices2)=(Zt.*et./ht./p0t.*A12+x.*A22)./rho(indices2);
end

%------------------------------------------------------------------
%alternative formulae for small |Z| (obtained by series expansions)
indices3=find(abs(Z)<=0.1);
if ~isempty(indices3)
Zt=Z(indices3);

Zt2=Zt.*Zt; Zt3=Zt2.*Zt; Zt4=Zt3.*Zt; Zt4=Zt3.*Zt; Zt5=Zt4.*Zt; Zt6=Zt5.*Zt;
Zt7=Zt6.*Zt;
p0t=p0(indices3);
S1=qw0(indices3).*p1(indices3)-p0t.*qw1(indices3); 
S2=qw0(indices3).*p2(indices3)-p0t.*qw2(indices3);
ht=h(indices3); ht2=ht.*ht;
ht3=ht2.*ht;ht4=ht3.*ht;ht5=ht4.*ht; ht6=ht5.*ht;
tp02=p0t.*p0t;tp03=p0t.*tp02;  tp04=tp03.*p0t; tp05=tp04.*p0t; 
p1t=p1(indices3); p2t=p2(indices3); qw1t=qw1(indices3); qw0t=qw0(indices3);
qw1t2=qw1t.*qw1(indices3);qw0t2=qw0t.*qw0t;
qw2t2=qw2(indices3).*qw2(indices3); p2t2=p2t.*p2t; p1t2=p1t.*p1t;
qw1qw2=qw1t.*qw2(indices3); qw2p1=qw2(indices3).*p1t; qw03=qw0t2.*qw0t; 
qw2p2=qw2(indices3).*p2t; p2p1=p2t.*p1t;

   T11=1+Zt/2+Zt2/24+Zt3/720+Zt4/40320+Zt5/3628800+Zt6/479001600+Zt7/87178291200;
   T12=(1+1/6*Zt+1/120*Zt2+1/5040*Zt3+1/362880*Zt4+1/39916800*Zt5+1/6227020800*Zt6+1/1307674368000*Zt7).*ht.*p0(indices3);
   T21=ht.*qw0(indices3).*(1+1/6*Zt+1/120*Zt2+1/5040*Zt3+1/362880*Zt4+1/39916800*Zt5+1/6227020800*Zt6); 
   T22=T11;

   %first two terms in Neumann series
   tmp=2/3*S1+(2/5*S1+2/15*S2).*Zt+(2/63*S2+4/63*S1).*Zt2+(2/405*S1+2/675*S2).*Zt3+(8/51975*S2+4/17325*S1).*Zt4+(4/773955*S2+4/552825*S1).*Zt5;
   A11=1+tmp/4;
   A22=1-tmp/4;
   A12=((1/93555*S1+1/135135*S2).*Zt4+(4/14175*S1+4/22275*S2).*Zt3+(1/378*S2+1/210*S1).*Zt2+(2/45*S1+2/105*S2).*Zt+1/6*S1+1/30*S2).*ht.*p0(indices3);
   A21=((-4/14175*S1-4/22275*S2).*Zt3+(-1/378*S2-1/210*S1).*Zt2+(-2/45*S1-2/105*S2).*Zt-1/6*S1-1/30*S2).*qw0(indices3).*ht;
   

   %second integral in Neumann series
   tmp=((464/467775*qw1t2+64/96525*qw1qw2+688/14189175*qw2t2).*tp05.*ht6+(-16/189*p2p1-68/1575*p2t2-4/105*p1t2).*p0t.*ht2+(-8/525*qw1t.*p1t-...
       128/22275*qw2p1+128/22275*p2t.*qw1t+8/1575*qw2p2).*tp03.*ht4).*qw03+((8/225*qw2p2+16/189*p2t.*qw1t-16/189*qw2p1-8/45*qw1t.*p1t).*tp02.*ht2+...
       (92/4725*qw1t2+16/1485*qw1qw2+4/5775*qw2t2).*tp04.*ht4-8/105*p2t2-16/105*p2p1-4/45*p1t2).*qw0t2+...
       ((16/189*qw1qw2+4/525*qw2t2+68/315*qw1t2).*tp03.*ht2+(64/105*p2t.*qw1t-64/105*qw2p1-8/9*qw1t.*p1t).*p0t).*qw0t+...
       (44/45*qw1t2+16/105*qw1qw2+8/105*qw2t2).*tp02+(-16/15*qw2p1+16/15*p2t.*qw1t)./ht2;   
   A11=A11+tmp/32;
   tmp=((8/225225*qw1t2+32/315315*qw1qw2+152/2627625*qw2t2).*p0t.^6.*ht.^7+(-8/315*p2p1-2/1155*p2t2-10/189*p1t2).*tp02.*ht3+...
       ((4/1485*p1t+16/17325*p2t).*qw1t-4/3861*qw2p2-16/17325*qw2p1).*tp04.*ht5).*qw03+(((8/189*p1t+4/225*p2t).*qw1t-4/225*qw2p1-8/693*qw2p2).*tp03.*ht3+...
       (-2/5*p1t2-2/105*p2t2-4/35*p2p1).*p0t.*ht+(38/32175*qw2t2+16/7425*qw1qw2+2/2475*qw1t2).*tp05.*ht5).*qw0t2+((2/189*qw1t2+8/315*qw1qw2+46/3465*qw2t2).*tp04.*ht3+...
       (-4/35*p2t2-4/5*p1t2)./ht+((4/21*p2t+12/35*p1t).*qw1t-4/105*qw2p2-4/21*qw2p1).*tp02.*ht).*qw0t+((4/5*p1t+4/5*p2t).*qw1t-4/5*qw2p1+4/35*qw2p2).*p0t./ht+...
       (2/35*qw2t2+2/35*qw1t2+4/35*qw1qw2).*tp03.*ht;
   A12=A12+tmp/24;
   tmp=(-14/96525*qw2t2-16/7425*qw1qw2-26/7425*qw1t2).*qw03.*tp04.*ht5+((-10/189*qw1t2-2/1155*qw2t2-8/315*qw1qw2).*qw0t2.*tp03+...
       (-8/693*qw2p2+4/225*qw2p1+(8/189*p1t-4/225*p2t).*qw1t).*qw03.*tp02).*ht3+...
       ((-2/5*qw1t2-2/105*qw2t2-4/35*qw1qw2).*qw0t.*tp02+((-4/21*p2t+12/35*p1t).*qw1t+4/21*qw2p1-4/105*qw2p2).*qw0t2.*p0t+(2/35*p2t2+4/35*p2p1+2/35*p1t2).*qw03).*ht+...
       ((-4/5*qw1t2-4/35*qw2t2).*p0t+(4/35*qw2p2+(-4/5*p2t+4/5*p1t).*qw1t+4/5*qw2p1).*qw0t)./ht;
   A21=A21+tmp/24;   
   tmp=((-16/66825*qw1t2-64/96525*qw1qw2-752/2027025*qw2t2).*tp05.*ht6+(16/189*p2p1+4/525*p2t2+68/315*p1t2).*p0t.*ht2+...
       ((-8/525*p1t-128/22275*p2t).*qw1t+128/22275*qw2p1+8/1575*qw2p2).*tp03.*ht4).*qw03+...
       (16/105*p2p1+((-8/45*p1t-16/189*p2t).*qw1t+8/225*qw2p2+16/189*qw2p1).*tp02.*ht2+...
       (-4/945*qw1t2-16/1485*qw1qw2-4/693*qw2t2).*tp04.*ht4+44/45*p1t2+8/105*p2t2).*qw0t2+...
       (((-64/105*p2t-8/9*p1t).*qw1t+64/105*qw2p1).*p0t+(-16/189*qw1qw2-68/1575*qw2t2-4/105*qw1t2).*tp03.*ht2).*qw0t+...
       (-4/45*qw1t2-16/105*qw1qw2-8/105*qw2t2).*tp02+(16/15*qw2p1-16/15*p2t.*qw1t)./ht2;
   A22=A22+tmp/32;  
   
sq=sqrt(T22.^2-2*T22.*T11+T11.^2+4*T12.*T21)/2;
eigv1=T11+sq; eigv2=T11-sq;
rho(indices3)=max(abs(eigv1),abs(eigv2));
Ts(1,1,indices3)=(T11.*A11+T12.*A21)./rho(indices3);
Ts(1,2,indices3)=(T11.*A12+T12.*A22)./rho(indices3);
Ts(2,1,indices3)=(T21.*A11+T22.*A21)./rho(indices3);
Ts(2,2,indices3)=(T21.*A12+T22.*A22)./rho(indices3);
end
end