function varargout=propagate_solution(part,E,y,imatch,forward)
%PROPAGATE_SOLUTION
% input: 
%   part: information about mesh and related data
%   E: eigenvalue
%   y: [y(x);yprime(x)]
%   imatch: index of matching point
%   forward: boolean
% output:
%  varargin{1}: y in matching point
%  varargin{2}: the Prufer phase

q0t=part.q0; p0t=part.p0; w0t=part.w0;
q1t=part.q1; p1t=part.p1; w1t=part.w1;
q2t=part.q2; p2t=part.p2; w2t=part.w2;

if nargout>1
 S=max(max((E*w0t-q0t)./p0t),1);
 S=sqrt(S);
  theta = prufer(y(1),y(2),y(1),y(2),(E*w0t(1)-q0t(1))*p0t(1),0,S);
  if forward && theta < 0
   theta = pi+theta; 
  elseif ~forward && theta<=0
   theta = pi+theta;
  end
  pruf=true;
else 
  pruf=false;
end

xs=part.xs;
if forward
  %from a to matching point  
  tt= 1:imatch;
  hs=xs(tt+1)-xs(tt);  
  if part.inf_a %infinite endpoint a
   hs(1)=-1/xs(2)+1/xs(1);    
  end
  if part.inf_b && imatch==length(part.p0) %matching point = infinite endpoint b
      hs(end)=-1/xs(imatch+1)+1/xs(imatch);    
  end
  % transfer matrices:
  Ts=get_propagation_matrix(E,hs,p0t(tt),p1t(tt),p2t(tt),q0t(tt),q1t(tt),q2t(tt),w0t(tt),w1t(tt),w2t(tt));
  
  for i=1:length(tt)
    T=Ts(:,:,i);
    y0=y;    y=T*y;
    if pruf
        ti=tt(i);
        [theta0,theta1]=prufer(y0(1),y0(2),y(1),y(2),(E*w0t(ti)-q0t(ti))*p0t(ti),hs(i),S);
        if i==1 && part.inda>0 && (theta1-theta0)/pi>5
             theta1=theta0;
        end
        theta=theta+theta1-theta0;
    end
  end
else
    %from b to matching point
  tt=length(p0t):-1:imatch+1;
  hs=xs(length(p0t)+1:-1:imatch+2)-xs(tt);
  if part.inf_b %infinite endpoint
   hs(1)=1/xs(length(p0t));    
  end
  Ts=get_propagation_matrix(E,hs,p0t(tt),p1t(tt),p2t(tt),q0t(tt),q1t(tt),q2t(tt),w0t(tt),w1t(tt),w2t(tt));
  for i=1:length(tt)
    T=Ts(:,:,i);
    y0=y; 
    iT(1,1)=T(2,2);  iT(1,2)=-T(1,2); iT(2,1)=-T(2,1);  iT(2,2)=T(1,1);   
    y=iT*y;
    if pruf
      ti=tt(i);
      [theta0,theta1]=prufer(y(1),y(2),y0(1),y0(2),(E*w0t(ti)-q0t(ti))*p0t(ti),hs(i),S);
      theta=theta-theta1+theta0;
    end
  end
end

varargout{1}=y;  
if pruf
    varargout{2}=theta;    
end  

end

function [theta0,theta1]=prufer(y0,y0p,y1,y1p,Epqw,h,S)
%PRUFER [theta0,theta1]=prufer(y0,y0p,y1,y1p,epqw,h,S)
%
% y0, y0p ,y1 ,y1p:     - the values of y and y' at the ends of an interval of stepsize h
%       v         :     - the constant approximation of the potential on that interval
%       e         :     - the energy
% theta0, theta1  :     - the values of the theta function corresponding to the scaled
%    prufer form (with scale factor S) at the two ends of the interval.

if Epqw > 0 %(i)  %p=1/P(x) !!!!    
    Si = sqrt(Epqw); %local scale factor+
    if y0p ~= 0
       theta0 = atan(Si*y0/y0p);
    elseif y0 > 0
         theta0 = pi/2;
    elseif y0 < 0
         theta0 = -pi/2;
    else
        theta0=0;
    end
    nphi = floor((theta0 + Si*h)/pi); 
    phibar = theta0 + Si*h - nphi*pi; %between -pi/2 and pi/2
    if y1p ~= 0
        phistar = atan(Si*y1/y1p);
    elseif y1 > 0
        phistar = pi/2;
    elseif y1 < 0
        phistar = -pi/2;
    else
        phistar = 0;
    end
    deltaphi = phistar - phibar;
    if deltaphi > pi/2
        deltaphi = deltaphi-pi; 
    elseif deltaphi < -pi/2
        deltaphi = deltaphi + pi;
    end
    theta1 = theta0 + Si*h + deltaphi;
    if theta1<theta0 && (abs(deltaphi+pi/2)<1e-6 ||  abs(theta1-theta0)>pi/20)%theta1*theta0<0  
            theta1=theta1+pi;
    end
    if Si == S
        return;
    end 
%   else rescaling procedure
    sigma = S/Si;   
    theta0 = theta0 + atan2((sigma-1)*sin(theta0)*cos(theta0),...
        1+(sigma-1)*sin(theta0)^2);
    theta1 = theta1 + atan2((sigma-1)*sin(theta1)*cos(theta1),...
        1+(sigma-1)*sin(theta1)^2);
    if theta1<theta0 && abs(theta1-theta0)>pi/25 %moet blijven voor transformed_hydrogen
            theta1=theta1+pi;
    end
    return;
%(ii): (e*w-q)*p <= 0  
elseif y0p ~= 0
    theta0 = atan(S*y0/y0p);
elseif y0 > 0 
    theta0 = pi/2;
elseif y0 < 0
    theta0 = -pi/2;
else
    theta0=0;
end
if y1p ~= 0
    theta1 = atan(S*y1/y1p);   %ylp is soms Inf
elseif y1 > 0 
    theta1 = pi/2;
elseif y1 < 0
    theta1 = -pi/2;
else
    theta1=0;
end
if y0*y1 >= 0 
    if theta0 > 0 && theta1 < 0
        theta1 = theta1 + pi;
    elseif theta0 < 0 && theta1 > 0
        theta1 = theta1 - pi;
    end
elseif theta0*theta1 > 0 
    theta1 = theta1 + pi;
end
end






