function woodssaxon_sing
%A Woods-Saxon problem with a singularity in 0
addpath(genpath(pwd))
addpath(genpath('./source'))

 
exact=[-48.951731624194039  -47.341691701825177  -45.237176986234736  -42.698002624727110 ...
     -39.767208069496149  -36.480106460118343 -32.868392986127205  -28.962621465368262 ...
      -24.794185466913213 -20.397447225237499 -15.812724871007093 -11.091535991112861 -6.308097192467161 -1.594465507689002];

 
tol=1e-8; %user input tolerance

print_messages(true);

problem.pfun=@pfun;  %function handles to p,q,w functions
problem.qfun=@qfun;
problem.wfun=@wfun;
problem.a=0;         %integration interval
problem.b=inf;
problem.bcs=[1 0 1 0]; %boundary conditions
problem.partition=[];

%compute eigenvalues
[E,partition]=calculate_eigenvalues(problem,0,13,tol,true);
disp('  k           E_k              exact error    estimated error')
for n=1:length(E.indices)
  disp(sprintf('%3d     %18.12f       %5.1E     %5.1E',E.indices(n),E.eigenvalues(n),...
      abs(E.eigenvalues(n)-exact(E.indices(n)+1)),abs(E.errors(n))));
end

number_of_meshpoints=length(partition.bp.xs)



%coefficient functions
function res=pfun(x)
res=1+0.*x;

function res=qfun(x)
t=exp((x-7)/0.6);
res=2./x.^2+(-50*(1-5*t./(3*(1+t)))./(1+t));
res(isnan(res))=0;

function res=wfun(x)
res=1+0.*x;
