function Collatz
%The Collatz problem
addpath(genpath(pwd))
addpath(genpath('./source'))


tol=1e-6; %user input tolerance

%exact eigenvalues
exact=64/9*(1:500).^2*pi^2;

print_messages(false);

problem.pfun=@pfun; %function handles to p,q,w functions
problem.qfun=@qfun;
problem.wfun=@wfun;
problem.a=1;      %integration interval
problem.b=2;
problem.bcs=[1 0 1 0]; %boundary conditions

%compute first 21 eigenvalues
[E,partition]=calculate_eigenvalues(problem,0,20,tol,true);
number_of_meshponts=length(partition.bp.xs)


%display results
disp('first 21 eigenvalues')
disp('  k           E_k              exact error    estimated error   relative error')
for n=1:length(E.indices)
  disp(sprintf('%3d     %18.12f       %5.1E       %5.1E         %5.1E',E.indices(n),E.eigenvalues(n),...
      abs(E.eigenvalues(n)-exact(E.indices(n)+1)),abs(E.errors(n)),...
      abs(E.eigenvalues(n)-exact(E.indices(n)+1))/exact(E.indices(n)+1)));
end

%compute first eigenfunction
[x,v,vp]=calculate_eigenfunction(partition,E.eigenvalues(1));
figure
k=1;
%exact eigenfunction
y1=(3/(8*k*pi)).*x.^(3/2).*sin(4*k*pi*(1-1./x.^2)/3);
[m,i]=max(v);
v=v.*y1(i)/v(i);
plot(x,y1,'b')
hold on
plot(x,v,'r*')
hold off
legend('exact y_{1}','y_{1} (tol=10^{-7})')
axis tight
error_in_eigenfunction=max(abs(y1-v))

%compute tenth eigenfunction
[x,v,vp]=calculate_eigenfunction(partition,E.eigenvalues(10));
figure
k=10;
y1=(3/(8*k*pi)).*x.^(3/2).*sin(4*k*pi*(1-1./x.^2)/3);
plot(x,y1,'b')
hold on
[m,i]=max(v);
v=v.*y1(i)/v(i);
plot(x,v,'r*')
hold off
legend('exact y_{10}','y_{10} (tol=10^{-7})')
axis tight
error_in_eigenfunction=max(abs(y1-v))

%compute twentieth eigenfunction
[x,v,vp]=calculate_eigenfunction(partition,E.eigenvalues(20));
figure
k=20;
y1=(3/(8*k*pi)).*x.^(3/2).*sin(4*k*pi*(1-1./x.^2)/3);
plot(x,y1,'b')
hold on
[m,i]=max(v);
v=v.*y1(i)/v(i);
plot(x,v,'r*')
hold off
legend('exact y_{1}','y_{1} (tol=10^{-7})')
axis tight
error_in_eigenfunction=max(abs(y1-v))





%coefficient functions
function res=pfun(x)
res=1;

function res=qfun(x)
res=3./4./x.^2;

function res=wfun(x)
res=1./x.^6;


