function Collatzb
addpath(genpath(pwd))
addpath(genpath('./source'))

tol=1e-8; %user input tolerance

%"exact" eigenvalues
exact=[0.07125047241552, ...
   0.33030839238097, ...
   0.75784187599800, ...
   1.35212218429719, ...
   2.11404669930052, ...
   3.04437343319217, ...
   4.14349398793540, ...
   5.41157878775638, ...
   6.84869557354521, ...
   8.45486973266229, ...
  10.23011034361830];

print_messages(false); %print_messages(true); to show messages during computation

problem.pfun=@pfun; %function handles to p,q,w functions
problem.qfun=@qfun;
problem.wfun=@wfun;
problem.a=-pi;      %integration interval
problem.b=pi;
problem.bcs=[1 0 1 0]; %boundary conditions
problem.partition=[];

%compute first 11 eigenvalues
[E,partition]=calculate_eigenvalues(problem,0,10,tol,true);
number_of_meshpoints=length(partition.bp.xs)


%display results
disp('The first 11 eigenvalues:')
disp('  k           E_k              exact error    estimated error   relative error')
for n=1:length(E.indices)
  disp(sprintf('%3d     %18.12f       %5.1E       %5.1E         %5.1E',E.indices(n),E.eigenvalues(n),...
      abs(E.eigenvalues(n)-exact(E.indices(n)+1)),abs(E.errors(n)),...
      abs(E.eigenvalues(n)-exact(E.indices(n)+1))/exact(E.indices(n)+1)));
end

% Compute eigenfunction y_10
[x,v,vp]=calculate_eigenfunction(partition,E.eigenvalues(10));
figure
plot(x,v)
hold on
plot(x,vp,'r')
hold off
legend('y_{10}','y''_{10}')
axis tight


%compute eigenvalues between 0 and 5
problem.partition=partition; %there is no need to recompute partition, so we pass it as input to calculate_eigenvalues
[E,partition]=calculate_eigenvalues(problem,0,5,tol,false);
number_of_meshponts=length(partition.bp.xs)


%display results
disp('Eigenvalues between 0 and 5:')
disp('  k           E_k              exact error    estimated error   relative error')
for n=1:length(E.indices)
  disp(sprintf('%3d     %18.12f       %5.1E       %5.1E         %5.1E',E.indices(n),E.eigenvalues(n),...
      abs(E.eigenvalues(n)-exact(E.indices(n)+1)),abs(E.errors(n)),...
      abs(E.eigenvalues(n)-exact(E.indices(n)+1))/exact(E.indices(n)+1)));
end




%coefficient functions
function res=pfun(x)
res=1;

function res=qfun(x)
res=0;

function res=wfun(x)
res=3+cos(x);


