function woodssaxon
%Woods-Saxon potential, Reference: Vanden Berghe, Fack and De Meyer, J.
%Comp. Appl. Math. 28 (1989)
addpath(genpath(pwd))
addpath(genpath('./source'))
a=0;
b=inf; %infinite integration interval

exact=[-49.45778872808258 ...
 -48.14843042000639 ...
 -46.29075395446623 ...
 -43.96831843181467 ...
 -41.23260777218090 ...
 -38.12278509672854 ...
 -34.67231320569997 ...
 -30.91224748790910 ... 
 -26.87344891605993 ...
 -22.58860225769320 ...
 -18.09468828212811 ...
 -13.43686904026007 ...
  -8.67608167074520 ...
  -3.90823248120989];

tol=1e-8; %user input tolerance

print_messages(true);

problem.pfun=@pfun; %function handles of p,q,w functions
problem.qfun=@qfun;
problem.wfun=@wfun;
problem.a=a;        %integration interval
problem.b=b;
problem.bcs=[1 0 1 0];  %boundary conditions
problem.partition=[];

%compute eigenvalues with indices 0-13
[E,partition]=calculate_eigenvalues(problem,0,13,tol,true);
disp('Eigenvalues E_0 to E_13')
disp('  k           E_k              exact error    estimated error')
for n=1:length(E.indices)
  disp(sprintf('%3d     %18.12f       %5.1E     %5.1E',E.indices(n),E.eigenvalues(n),...
      abs(E.eigenvalues(n)-exact(E.indices(n)+1)),abs(E.errors(n))));
end

%compute eigenvalues between -50 and 0
[E,partition]=calculate_eigenvalues(problem,-50,0,tol,false);
disp('Eigenvalues between -50 and 0')
disp('  k           E_k              exact error    estimated error')
for n=1:length(E.indices)
  disp(sprintf('%3d     %18.12f       %5.1E     %5.1E',E.indices(n),E.eigenvalues(n),...
      abs(E.eigenvalues(n)-exact(E.indices(n)+1)),abs(E.errors(n))));
end


number_of_meshpoints=length(partition.bp.xs)


function res=pfun(x)
res=1+0.*x;

function res=qfun(x)
t=exp((x-7)/0.6);
res=-50*(1-5*t./(3*(1+t)))./(1+t);
%Be careful with potential over/underflows
if x>400
    res=0;
end
res(isnan(res))=0;

function res=wfun(x)
res=1+0.*x;
